# Migrate Spreadsheet

## Overview

The module provides a migrate source plugin for importing data from spreadsheet files. This source plugin uses the [PhpOffice/PhpSpreadsheet](https://github.com/PHPOffice/PhpSpreadsheet) library to read from the spreadsheet files.

[The supported source files](https://github.com/PHPOffice/PhpSpreadsheet#file-formats-supported) includes .ods, .xls, .xlsx, .csv.

## Installing PHPSpreadsheet

Usually we use [Composer](https://getcomposer.org/) to install dependencies in Drupal 8. Unfortunately, the `composer.json` file provided in the module root directory is not automatically parsed when the module gets enabled. For this reason, before installing the module, relevant lines from that file should be copied in the project top `composer.json` file. The entries under `"repositories"`, `"require"` and `"autoload"` > `"psr-4"` should be copied under the same arrays, in the top project's `composer.json`. Then a `$ composer update` should be issued.

## Usage

In your migration file configure the source plugin as follows:

```yaml
source:
  plugin: spreadsheet

  # The source file. The path can be either relative to Drupal root but it can
  # be a also an absolute reference such as a stream wrapper.
  file: ../resources/source_file.xlsx
  # The worksheet to be read.
  worksheet: 'Personnel list'

  # The top-left cell where date area starts (excluding the header, if exists).
  # It should use a spreadsheet representation (B2, A2, ...). The data area does
  # not include the header. If it's missed, the assumption is that the first row
  # contains the table header and the data origin is the first cell of the
  # second row. That is A2. In this example the data area starts from the second
  # column of the third row.
  origin: B3

  # The row where the header is placed, if any. If missed, there's no table
  # header for the data being inserted and the spreadsheet columns (A, B, ...)
  # the table header. If the table row is the first row, this should be 1. The
  # will be used as column names. The value of 2 means that the table header is
  # on the second row.
  header_row: 2

  # The list of columns to be returned. Is basically a list of table header cell
  # values if a header has been defined (see `header_row`). If there's no table
  # header (i.e. `header_row` is missing), it should contain a list/sequence of
  # column letters (A, B, C, ...).
  columns:
    - ID
    - Revision
    - 'First name'
    - 'Sure name'
    - Gender

  # The name of the column with the row index. If this setting is specified, the
  # source will return also a column with this name containing the row index.
  # In this example 'Row no.' can be used later in `keys:` list to make this
  # column a primary key column.
  row_index_column: 'Row no.'

  # The primary key as list of keys. Is a list of source columns that are
  # composing the primary key. The list is keyed by column name and has the
  # field schema as value. If the table have a header (i.e. `header_row` is set)
  # the keys will be set ad the name of header cells acting as primary index.
  # Otherwise the column letters (A, B, C, ...) can be used. If no keys are
  # defined, the current row position will be returned as primary key, but in
  # this case, 'row_index_column' should have a value.
  keys:
    ID:
      type: integer
    Revision:
      type: string
```

## Author

Claudiu Cristea ([claudiu.cristea](https://www.drupal.org/u/claudiu.cristea))
